"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __values = (this && this.__values) || function(o) {
    var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
    if (m) return m.call(o);
    if (o && typeof o.length === "number") return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
    throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
};
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArgsParser = exports.ParsedArgumentsCollection = void 0;
var exceptions_1 = require("./exceptions");
var utils_1 = require("./utils/utils");
var help_1 = require("./utils/help");
/**
 * A collection of parsed arguments.
 *
 * @category Classes
 */
var ParsedArgumentsCollection = /** @class */ (function () {
    function ParsedArgumentsCollection() {
        /**
         * A collection of parsed positional arguments.
         */
        this.positionalArgs = {};
        /**
         * A collection of parsed optional arguments.
         */
        this.optionalArgs = {};
    }
    Object.defineProperty(ParsedArgumentsCollection.prototype, "positional", {
        /**
         * Returns all arguments as a record.
         */
        get: function () {
            return __assign({}, this.positionalArgs);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(ParsedArgumentsCollection.prototype, "optional", {
        /**
         * Returns all arguments as a record.
         */
        get: function () {
            return __assign({}, this.optionalArgs);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Adds a new argument to the collection.
     * @param name - The name of the argument.
     * @param value - The value of the argument.
     * @returns The instance of ParsedArgumentsCollection for chaining.
     */
    ParsedArgumentsCollection.prototype.add = function (name, value) {
        var formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            this.positionalArgs[formattedName] = value;
        }
        else {
            this.optionalArgs[formattedName] = value;
        }
        return this;
    };
    /**
     * Retrieves an argument value.
     * @param name - The name of the argument.
     * @returns The value of the argument, or undefined if not found.
     */
    ParsedArgumentsCollection.prototype.get = function (name) {
        var _a, _b;
        var formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            return ((_a = this.positional[formattedName]) !== null && _a !== void 0 ? _a : undefined);
        }
        return ((_b = this.optionalArgs[formattedName]) !== null && _b !== void 0 ? _b : undefined);
    };
    /**
     * Checks if an argument exists in the collection.
     * @param name - The name of the argument.
     * @returns True if the argument exists, otherwise false.
     */
    ParsedArgumentsCollection.prototype.has = function (name) {
        var _a, _b;
        var formattedName = this.formatName(name);
        if (this.isPositional(name)) {
            return ((_a = this.positional[formattedName]) !== null && _a !== void 0 ? _a : undefined) !== undefined;
        }
        return ((_b = this.optionalArgs[formattedName]) !== null && _b !== void 0 ? _b : undefined) !== undefined;
    };
    /**
     * Formats the argument name by removing leading dashes.
     * @param name - The argument name.
     * @returns The formatted name.
     */
    ParsedArgumentsCollection.prototype.formatName = function (name) {
        return name.replace(/^--/, '');
    };
    /**
     * Checks if the argument is positional.
     * @param name - The argument name.
     * @returns True if the argument is positional, otherwise false.
     */
    ParsedArgumentsCollection.prototype.isPositional = function (name) {
        return !name.startsWith('--');
    };
    return ParsedArgumentsCollection;
}());
exports.ParsedArgumentsCollection = ParsedArgumentsCollection;
/**
 * Parser for command-line arguments.
 *
 * @category Classes
 */
var ArgsParser = /** @class */ (function () {
    /**
     * Constructs an instance of ArgsParser with the provided argument configurations.
     * @param args - An array of argument configurations.
     */
    function ArgsParser(args) {
        var e_1, _a;
        if (args === void 0) { args = []; }
        /**
         * Maps argument names to their configuration.
         */
        this.argsMap = new Map();
        /**
         * Maps argument aliases to their corresponding names.
         */
        this.aliasMap = new Map();
        /**
         * Tracks used argument names and aliases to prevent duplicates.
         */
        this.usedArgs = new Set();
        try {
            for (var args_1 = __values(args), args_1_1 = args_1.next(); !args_1_1.done; args_1_1 = args_1.next()) {
                var arg = args_1_1.value;
                this.addArgument(arg);
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (args_1_1 && !args_1_1.done && (_a = args_1.return)) _a.call(args_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    }
    Object.defineProperty(ArgsParser.prototype, "help", {
        /**
         * Retrieves the help message.
         */
        get: function () {
            var e_2, _a, e_3, _b;
            var positionalArgs = this.getPositionalArguments();
            var optionalArgs = this.getOptionalArguments();
            var positionalArgsRows = [];
            try {
                for (var positionalArgs_1 = __values(positionalArgs), positionalArgs_1_1 = positionalArgs_1.next(); !positionalArgs_1_1.done; positionalArgs_1_1 = positionalArgs_1.next()) {
                    var arg = positionalArgs_1_1.value;
                    var nargsConfig = (0, utils_1.buildNArgsConfig)(arg);
                    positionalArgsRows.push.apply(positionalArgsRows, __spreadArray([], __read((0, help_1.formatArgHelp)(arg, nargsConfig)), false));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (positionalArgs_1_1 && !positionalArgs_1_1.done && (_a = positionalArgs_1.return)) _a.call(positionalArgs_1);
                }
                finally { if (e_2) throw e_2.error; }
            }
            var optionalArgsRows = [];
            try {
                for (var optionalArgs_1 = __values(optionalArgs), optionalArgs_1_1 = optionalArgs_1.next(); !optionalArgs_1_1.done; optionalArgs_1_1 = optionalArgs_1.next()) {
                    var arg = optionalArgs_1_1.value;
                    var nargsConfig = (0, utils_1.buildNArgsConfig)(arg);
                    optionalArgsRows.push.apply(optionalArgsRows, __spreadArray([], __read((0, help_1.formatArgHelp)(arg, nargsConfig)), false));
                }
            }
            catch (e_3_1) { e_3 = { error: e_3_1 }; }
            finally {
                try {
                    if (optionalArgs_1_1 && !optionalArgs_1_1.done && (_b = optionalArgs_1.return)) _b.call(optionalArgs_1);
                }
                finally { if (e_3) throw e_3.error; }
            }
            var result = [];
            if (positionalArgsRows.length > 0) {
                result.push.apply(result, __spreadArray([['Positional arguments:'],
                    ['']], __read((0, help_1.tabTableRows)(positionalArgsRows)), false));
            }
            if (optionalArgsRows.length > 0) {
                result.push.apply(result, __spreadArray([['Optional arguments:'],
                    ['']], __read((0, help_1.tabTableRows)(optionalArgsRows)), false));
            }
            return (0, help_1.convertToTable)(result, 1);
        },
        enumerable: false,
        configurable: true
    });
    /**
     * Adds the help message for the arguments to the parser.
     * @returns The updated parser.
     */
    ArgsParser.prototype.addHelp = function () {
        return this.addArgument({
            name: '--help',
            alias: '-h',
            description: 'Show help',
            type: 'boolean',
            const: true,
            default: false,
        });
    };
    /**
     * Adds a new argument configuration to the parser.
     * @param config - The argument configuration.
     * @returns The instance of ArgsParser for chaining.
     */
    ArgsParser.prototype.addArgument = function (config) {
        this.checkArgumentConfig(config);
        this.argsMap.set(config.name, config);
        this.usedArgs.add(config.name);
        if (config.alias !== undefined) {
            this.aliasMap.set(config.alias, config.name);
            this.usedArgs.add(config.alias);
        }
        return this;
    };
    /**
     * Parses the given argument string and returns a collection of parsed arguments.
     * @param argv - The argument string.
     * @returns A ParsedArgumentsCollection containing the parsed arguments.
     */
    ArgsParser.prototype.parse = function (argv) {
        var e_4, _a, e_5, _b, e_6, _c;
        var positionalArgs = this.getPositionalArguments();
        var optionalArgs = new Set(this.getOptionalArguments());
        var _d = __read((0, utils_1.parseArgsArray)(argv), 2), parsedPositional = _d[0], parsedOptional = _d[1];
        parsedPositional.reverse();
        var result = new ParsedArgumentsCollection();
        try {
            for (var positionalArgs_2 = __values(positionalArgs), positionalArgs_2_1 = positionalArgs_2.next(); !positionalArgs_2_1.done; positionalArgs_2_1 = positionalArgs_2.next()) {
                var argConfig = positionalArgs_2_1.value;
                var nargsConfig = (0, utils_1.buildNArgsConfig)(argConfig);
                this.checkEnoughPositionalValues(parsedPositional, argConfig.name, nargsConfig);
                var toReadCount = this.getArgsCountToRead(nargsConfig, parsedPositional.length);
                var value = [];
                for (var i = 0; i < toReadCount; ++i) {
                    if (parsedPositional.length === 0) {
                        break;
                    }
                    value.push(parsedPositional.pop());
                }
                var castedValue = (0, utils_1.castArgValue)(value, argConfig, nargsConfig, value.length > 0);
                result.add(argConfig.name, (0, utils_1.validateCastedArgValue)(castedValue, argConfig, nargsConfig));
            }
        }
        catch (e_4_1) { e_4 = { error: e_4_1 }; }
        finally {
            try {
                if (positionalArgs_2_1 && !positionalArgs_2_1.done && (_a = positionalArgs_2.return)) _a.call(positionalArgs_2);
            }
            finally { if (e_4) throw e_4.error; }
        }
        if (parsedPositional.length > 0) {
            throw new exceptions_1.ArgumentValueError('Too many positional arguments.');
        }
        try {
            for (var _e = __values(Object.entries(parsedOptional)), _f = _e.next(); !_f.done; _f = _e.next()) {
                var _g = __read(_f.value, 2), key = _g[0], value = _g[1];
                var argConfig = this.getArgConfig(key);
                var nargsConfig = (0, utils_1.buildNArgsConfig)(argConfig);
                var castedValue = (0, utils_1.castArgValue)(value, argConfig, nargsConfig);
                result.add(argConfig.name, (0, utils_1.validateCastedArgValue)(castedValue, argConfig, nargsConfig));
                optionalArgs.delete(argConfig);
            }
        }
        catch (e_5_1) { e_5 = { error: e_5_1 }; }
        finally {
            try {
                if (_f && !_f.done && (_b = _e.return)) _b.call(_e);
            }
            finally { if (e_5) throw e_5.error; }
        }
        try {
            for (var optionalArgs_2 = __values(optionalArgs), optionalArgs_2_1 = optionalArgs_2.next(); !optionalArgs_2_1.done; optionalArgs_2_1 = optionalArgs_2.next()) {
                var argConfig = optionalArgs_2_1.value;
                var nargsConfig = (0, utils_1.buildNArgsConfig)(argConfig);
                var castedValue = (0, utils_1.castArgValue)([], argConfig, nargsConfig, false);
                result.add(argConfig.name, (0, utils_1.validateCastedArgValue)(castedValue, argConfig, nargsConfig));
            }
        }
        catch (e_6_1) { e_6 = { error: e_6_1 }; }
        finally {
            try {
                if (optionalArgs_2_1 && !optionalArgs_2_1.done && (_c = optionalArgs_2.return)) _c.call(optionalArgs_2);
            }
            finally { if (e_6) throw e_6.error; }
        }
        return result;
    };
    /**
     * Checks the validity of an argument configuration.
     * @param config - The argument configuration.
     * @throws AddArgumentError if the configuration is invalid.
     */
    ArgsParser.prototype.checkArgumentConfig = function (config) {
        if (config.name.startsWith('-') && !config.name.startsWith('--')) {
            throw new exceptions_1.AddArgumentError("Argument name ".concat(config.name, " is invalid."));
        }
        if (config.alias !== undefined && (!config.alias.startsWith('-') || config.alias.startsWith('--'))) {
            throw new exceptions_1.AddArgumentError("Argument name ".concat(config.name, " is invalid."));
        }
        if (this.usedArgs.has(config.name)) {
            throw new exceptions_1.AddArgumentError("Argument ".concat(config.name, " already exists."));
        }
        if (config.alias !== undefined && this.usedArgs.has(config.alias)) {
            throw new exceptions_1.AddArgumentError("Argument ".concat(config.alias, " already exists."));
        }
    };
    /**
     * Checks the number of positional arguments.
     * @param values - An array of positional argument values.
     * @param argName - The name of the argument.
     * @param nargsConfig - The nargs configuration.
     */
    ArgsParser.prototype.checkEnoughPositionalValues = function (values, argName, nargsConfig) {
        var errorMessage = "Not enough positional arguments. ".concat(argName, " is required.");
        if (!nargsConfig.multiple) {
            if (!nargsConfig.allowEmpty && values.length === 0) {
                throw new exceptions_1.ArgumentValueError(errorMessage);
            }
            return;
        }
        if (!nargsConfig.allowEmpty && values.length === 0) {
            throw new exceptions_1.ArgumentValueError(errorMessage);
        }
        if (!nargsConfig.allowEmpty && nargsConfig.count !== undefined && nargsConfig.count > values.length) {
            throw new exceptions_1.ArgumentValueError(errorMessage);
        }
    };
    ArgsParser.prototype.getArgsCountToRead = function (nargsConfig, totalCount) {
        if (!nargsConfig.multiple) {
            return 1;
        }
        if (nargsConfig.count !== undefined) {
            return nargsConfig.count;
        }
        return totalCount;
    };
    /**
     * Retrieves the argument configuration for a given key.
     * @param key - The argument key.
     * @returns The corresponding ArgConfig.
     * @throws ArgumentNameError if the argument key is unknown.
     */
    ArgsParser.prototype.getArgConfig = function (key) {
        var _a;
        var config = this.argsMap.get((_a = this.aliasMap.get(key)) !== null && _a !== void 0 ? _a : key);
        if (config === undefined) {
            throw new exceptions_1.ArgumentNameError("Unknown argument ".concat(key, "."));
        }
        return config;
    };
    /**
     * Retrieves the positional arguments.
     * @returns An array of positional argument configurations.
     */
    ArgsParser.prototype.getPositionalArguments = function () {
        var _this = this;
        return __spreadArray([], __read(this.argsMap.values()), false).filter(function (x) { return !_this.isOptional(x); });
    };
    /**
     * Retrieves the optional arguments.
     * @returns An array of optional argument configurations.
     */
    ArgsParser.prototype.getOptionalArguments = function () {
        var _this = this;
        return __spreadArray([], __read(this.argsMap.values()), false).filter(function (x) { return _this.isOptional(x); });
    };
    /**
     * Checks if an argument is optional.
     * @param config - The argument configuration.
     * @returns True if the argument is optional, otherwise false.
     */
    ArgsParser.prototype.isOptional = function (config) {
        return config.name.startsWith('--');
    };
    return ArgsParser;
}());
exports.ArgsParser = ArgsParser;
//# sourceMappingURL=classes.js.map