"use strict";
var __read = (this && this.__read) || function (o, n) {
    var m = typeof Symbol === "function" && o[Symbol.iterator];
    if (!m) return o;
    var i = m.call(o), r, ar = [], e;
    try {
        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);
    }
    catch (error) { e = { error: error }; }
    finally {
        try {
            if (r && !r.done && (m = i["return"])) m.call(i);
        }
        finally { if (e) throw e.error; }
    }
    return ar;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseArgsArray = parseArgsArray;
exports.castArgValue = castArgValue;
exports.validateCastedArgValue = validateCastedArgValue;
exports.formatArgNameWithAlias = formatArgNameWithAlias;
exports.buildNArgsConfig = buildNArgsConfig;
var exceptions_1 = require("../exceptions");
function parseArgsArray(argv) {
    var foundIndex = argv.findIndex(function (x) { return x.startsWith('-'); });
    var optionalBegin = foundIndex !== -1 ? foundIndex : argv.length;
    var positional = __spreadArray([], __read(argv.slice(0, optionalBegin)), false);
    var optionalBuffer = __spreadArray([], __read(argv.slice(optionalBegin)), false).reverse();
    var optional = {};
    var argName = undefined;
    var argValues = [];
    while (optionalBuffer.length > 0) {
        var item = optionalBuffer.pop();
        if (item.match(/^-[a-zA-Z0-9]{2,}$/)) {
            var items = item.slice(1).split('').reverse();
            optionalBuffer.push.apply(optionalBuffer, __spreadArray([], __read(items.map(function (x) { return "-".concat(x); })), false));
            continue;
        }
        if (item.startsWith('-')) {
            if (argName !== undefined) {
                optional[argName] = argValues;
            }
            argName = item;
            argValues = [];
        }
        else {
            argValues.push(item);
        }
    }
    if (argName !== undefined) {
        optional[argName] = argValues;
    }
    return [positional, optional];
}
/**
 * Casts the given value to the type of the argument configuration.
 *
 * @param value - The value to cast.
 * @param argConfig - The argument configuration.
 * @param nargsConfig - The nargs configuration.
 * @param isset - Whether the argument is set. Defaults to true.
 * @param recursive - Whether this is a recursive call. Defaults to false.
 *
 * @returns The casted value.
 *
 * @category Utils
 */
function castArgValue(value, argConfig, nargsConfig, isset, recursive) {
    var _a, _b;
    if (isset === void 0) { isset = true; }
    if (recursive === void 0) { recursive = false; }
    var multiple = recursive ? false : nargsConfig.multiple;
    if (!isset) {
        if (multiple) {
            return (_a = argConfig.default) !== null && _a !== void 0 ? _a : [];
        }
        return argConfig.default;
    }
    if (multiple) {
        var result = value
            .filter(function (x) { return x !== ''; })
            .map(function (v) { return castArgValue([v], argConfig, nargsConfig, isset, true); });
        return result.length > 0 ? result : ((_b = argConfig.const) !== null && _b !== void 0 ? _b : []);
    }
    var _value;
    if (value.length === 1) {
        _value = value[0];
    }
    else if (value.length === 0) {
        _value = argConfig.const;
    }
    else {
        throw new exceptions_1.ArgumentValueError("Too many values for single argument: ".concat(formatArgNameWithAlias(argConfig)));
    }
    if (_value === undefined) {
        return _value;
    }
    switch (argConfig.type) {
        case 'string': return _value;
        case 'number': return Number(_value);
        case 'boolean': return typeof _value === 'boolean' ? _value : _value !== 'false' && _value !== '0';
    }
}
/**
 * Validates the casted argument value.
 *
 * @param value - The casted argument value.
 * @param argConfig - The argument configuration.
 * @param nargsConfig - The nargs configuration.
 *
 * @returns The validated value.
 *
 * @throws ArgumentValueError if the value is invalid.
 *
 * @category Utils
 */
function validateCastedArgValue(value, argConfig, nargsConfig) {
    if (argConfig.choices !== undefined) {
        if (!nargsConfig.multiple && !argConfig.choices.includes(value)) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat(formatArgNameWithAlias(argConfig), " value must be one of ").concat(argConfig.choices.join(', '), "."));
        }
        else if (nargsConfig.multiple && !value.every(function (x) { return argConfig.choices.includes(x); })) {
            throw new exceptions_1.ArgumentValueError("Argument ".concat(formatArgNameWithAlias(argConfig), " values must be some of ").concat(argConfig.choices.join(', '), "."));
        }
    }
    if (argConfig.validator !== undefined && !argConfig.validator(value)) {
        throw new exceptions_1.ArgumentValueError("Argument ".concat(formatArgNameWithAlias(argConfig), " value is invalid."));
    }
    return value;
}
/**
 * Formats the argument name with alias.
 *
 * @param argConfig - The argument configuration.
 *
 * @returns The formatted argument name.
 *
 * @category Utils
 */
function formatArgNameWithAlias(argConfig) {
    return "".concat(argConfig.name).concat(argConfig.alias ? " (".concat(argConfig.alias, ")") : '');
}
/**
 * Builds the NArgs configuration from an argument configuration.
 *
 * @param argConfig - The argument configuration.
 *
 * @returns The NArgs configuration.
 *
 * @category Utils
 */
function buildNArgsConfig(argConfig) {
    var multiple = argConfig.nargs === '*' || argConfig.nargs === '+' || typeof argConfig.nargs === 'number';
    var allowEmpty = argConfig.nargs === '*' || argConfig.nargs === '?' || argConfig.default !== undefined;
    var count = typeof argConfig.nargs === 'number' ? argConfig.nargs : undefined;
    return { multiple: multiple, allowEmpty: allowEmpty, count: count };
}
//# sourceMappingURL=utils.js.map